# Generated by Django 5.1.3 on 2025-01-08 22:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('posts', '0004_nonstandardidpost'),
    ]

    operations = [
        migrations.CreateModel(
            name='UuidIdPost',
            fields=[
                ('id', models.UUIDField(primary_key=True, serialize=False)),
                ('title', models.CharField(max_length=255)),
                ('body', models.TextField()),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
