# Generated by Django 4.2.6 on 2024-10-24 02:20

from django.db import migrations, models
import posts.models


class Migration(migrations.Migration):
    dependencies = [
        ("posts", "0003_postnogeo"),
    ]

    operations = [
        migrations.CreateModel(
            name="NonStandardIdPost",
            fields=[
                (
                    "crazy_id",
                    models.CharField(
                        default=posts.models.rand_id,
                        max_length=128,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("title", models.CharField(max_length=255)),
                ("body", models.TextField()),
            ],
            options={
                "verbose_name": "NonStandard Id Post",
                "verbose_name_plural": "NonStandard IdPosts",
            },
        ),
    ]
