# Generated by Django 4.2.6 on 2024-05-14 13:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('posts', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='post',
            name='lat',
            field=models.FloatField(default=0.0),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='post',
            name='lng',
            field=models.FloatField(default=0.0),
            preserve_default=False,
        ),
    ]
